@@ -14,25 +14,17 @@
 ## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-genFunAmh <- function(copula, u) {
+iPsiAmh <- function(copula, u) {
   alpha <- copula@parameters[1]
   log((1 - alpha * (1 - u)) / u)
 }
 
-genInvAmh <- function(copula, s) {
+psiAmh <- function(copula, s) {
   alpha <- copula@parameters[1]
   (1 - alpha) / (exp(s) - alpha)
 }
 
-genFunDer1Amh <- function(copula, u) {
-  eval(amhCopula.genfunDer.expr[1], list(u=u, alpha=copula@parameters[1]))
-}
-genFunDer2Amh <- function(copula, u) {
-  eval(amhCopula.genfunDer.expr[2], list(u=u, alpha=copula@parameters[1]))
-}
-
-
-amhCopula <- function(param, dim = 2L) {
+amhCopula <- function(param = NA_real_, dim = 2L) {
   ## get expressions of cdf and pdf
   cdfExpr <- function(n) {
     expr <-   "log((1 - alpha * (1 - u1)) / u1)"
@@ -67,13 +59,13 @@ amhCopula <- function(param, dim = 2L) {
       param.names = "param",
       param.lowbnd = -1,
       param.upbnd = 1,
-      message = "Amh copula family; Archimedean copula")
+      fullname = "Amh copula family; Archimedean copula")
 }
 
 
 pamhCopula <- function(copula, u) {
   dim <- copula@dimension
-  if(!is.matrix(u)) u <- matrix(u, ncol = dim)
+  ## if(!is.matrix(u)) u <- matrix(u, ncol = dim)
   alpha <- copula@parameters[1]
   if (abs(alpha) <= .Machine$double.eps^.9) return (apply(u, 1, prod))
   ## for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
@@ -83,7 +75,7 @@ pamhCopula <- function(copula, u) {
 }
 
 
-damhCopula <- function(copula, u, log = FALSE, ...) {
+damhCopula <- function(u, copula, log = FALSE, ...) {
   dim <- copula@dimension
   if(!is.matrix(u)) u <- matrix(u, ncol = dim)
   alpha <- copula@parameters[1]
@@ -98,7 +90,7 @@ damhCopula <- function(copula, u, log = FALSE, ...) {
 }
 
 
-ramhCopula <- function(copula, n) {
+ramhCopula <- function(n, copula) {
   dim <- copula@dimension
   alpha <- copula@parameters[1]
   val <- matrix(runif(n * dim), nrow = n)
@@ -118,32 +110,70 @@ ramhCopula <- function(copula, n) {
   cbind(u, v)
 }
 
-kendallsTauAmhCopula <- function(copula) {
+tauAmhCopula <- function(copula) {
   alpha <- copula@parameters[1]
   ## Nelsen (2006, p.172)
   ## range of tau: [(5 - 8 log 2) / 3, 1/3] ~= [-0.1817, 0.3333]
   (3 * alpha - 2) / 3 / alpha - 2 / 3 * (1 - 1/alpha)^2 * log(1 - alpha)
 }
 
-## spearmansRhoAmhCopula <- function(copula, ...) {
-##   alpha <- copula@parameters[1]
-##   ## Nelsen (2006, p.172); need dilog function
-##   ## range of rho: 33 - 48 log 2, 4 pi^2 - 39] ~= [-0.2711, 0.4784]
-##   12 * (1 + alpha) / alpha^2 * dilog(1 - alpha) - 24 * (1 - alpha) / alpha^2 * log(1 - alpha) - 3 * (alpha + 12) / alpha
-## }
+rhoAmhCopula <- function(copula) {
+  alpha <- copula@parameters[1]
+  ## Nelsen (2006, p.172); need dilog function, where his dilog(x) = Li_2(1-x) = polylog(1-x, 2)
+  ## range of rho: 33 - 48 log 2, 4 pi^2 - 39] ~= [-0.2711, 0.4784]
+  12 * (1 + alpha) / alpha^2 * polylog(alpha,2) - 24 * (1 - alpha) / alpha^2 * log1p(- alpha) -
+      3 * (alpha + 12) / alpha
+}
 
-setMethod("rcopula", signature("amhCopula"), ramhCopula)
-setMethod("pcopula", signature("amhCopula"), pamhCopula)
-setMethod("dcopula", signature("amhCopula"), damhCopula)
+pMatAmh <- function (u, copula, ...) {
+    ## was pamhCopula
+    stopifnot(ncol(u) == (d <- copula@dimension))
+    th <- copula@parameters
+    if(d == 2 && !copAMH@paraConstr(th)) # for now, .. to support negative tau
+        pamhCopula(copula, u)
+    else
+        pacopula(u, copAMH, theta=copula@parameters, ...)
+}
 
-setMethod("genFun", signature("amhCopula"), genFunAmh)
-setMethod("genInv", signature("amhCopula"), genInvAmh)
-setMethod("genFunDer1", signature("amhCopula"), genFunDer1Amh)
-setMethod("genFunDer2", signature("amhCopula"), genFunDer2Amh)
+dMatAmh <- function (u, copula, log = FALSE, ...) {
+    ## was  damhCopula
+    stopifnot(ncol(u) == (d <- copula@dimension))
+    th <- copula@parameters
+    if(d == 2 && !copAMH@paraConstr(th)) # for now, .. to support negative tau
+        damhCopula(u, copula, log=log)
+    else
+        copAMH@dacopula(u, theta=copula@parameters, log=log, ...)
+}
 
-setMethod("kendallsTau", signature("amhCopula"), kendallsTauAmhCopula)
-## setMethod("spearmansRho", signature("amhCopula"), spearmansRhoAmhCopula)
-## setMethod("tailIndex", signature("amhCopula"), tailIndexAmhCopula)
+setMethod("rCopula", signature("numeric", "amhCopula"), ramhCopula)
 
-setMethod("calibKendallsTau", signature("amhCopula"), calibKendallsTauCopula)
+setMethod("pCopula", signature("numeric", "amhCopula"),
+	  function (u, copula, ...)
+          pMatAmh(matrix(u, ncol = dim(copula)), copula, ...))
+setMethod("pCopula", signature("matrix", "amhCopula"), pMatAmh)
+
+setMethod("dCopula", signature("numeric", "amhCopula"),
+	  function (u, copula, ...)
+          dMatAmh(matrix(u, ncol = dim(copula)), copula, ...))
+setMethod("dCopula", signature("matrix", "amhCopula"), dMatAmh)
+
+setMethod("iPsi", signature("amhCopula"), iPsiAmh)
+## setMethod("iPsi", signature("amhCopula"),
+## 	  function(copula, u) copAMH@iPsi(u, theta=copula@parameters))
+setMethod("psi", signature("amhCopula"), psiAmh)
+## FIXME
+## setMethod("psi", signature("amhCopula"),
+## 	  function(copula, s) copAMH@psi(t=s, theta=copula@parameters))
+setMethod("diPsi", signature("amhCopula"),
+	  function(copula, u, degree=1, log=FALSE, ...) {
+              s <- if(log || degree %% 2 == 0) 1. else -1.
+              s* copAMH@absdiPsi(u, theta=copula@parameters, degree=degree, log=log, ...)
+      })
+
+
+setMethod("tau", signature("amhCopula"), tauAmhCopula)
+setMethod("rho", signature("amhCopula"), rhoAmhCopula)
+setMethod("tailIndex", signature("amhCopula"), function(copula) c(lower=0, upper=0))
+
+setMethod("iTau", signature("amhCopula"), iTauCopula)
 
