@@ -14,9 +14,18 @@
 ## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-### Test of exchangeability based on An ########################################
-
-exchEVTest <- function(x, N = 1000,  estimator = "CFG", derivatives = "Cn", m = 100)
+##' Test of exchangeability for bivariate EV copulas based on the Pickands
+##' or CFG estimators -- see SJS paper
+##'
+##' @title Test of exchangeability based on An
+##' @param x the data
+##' @param N number of multiplier replications
+##' @param estimator Pickands or CFG
+##' @param derivatives based on "An" or "Cn"
+##' @param m grid size
+##' @return an object of class 'htest'
+##' @author Ivan Kojadinovic
+exchEVTest <- function(x, N = 1000, estimator = "CFG", derivatives = "Cn", m = 100)
 {
   ## make pseudo-observations
   n <- nrow(x)
@@ -56,20 +65,23 @@ exchEVTest <- function(x, N = 1000,  estimator = "CFG", derivatives = "Cn", m =
              as.integer(N),
              s0 = double(N))$s0
 
-  structure(class = "exchTest",
-	    list(statistic=s, pvalue=(sum(s0 >= s)+0.5)/(N+1)))
+  structure(class = "htest",
+	    list(method = paste("Test of exchangeability for bivariate extreme-value copulas with argument 'estimator' set to '",
+                 estimator,"', argument 'derivatives' set to '",derivatives,"' and argument 'm' set to ", m, sep=""),
+                 statistic = c(statistic = s),
+                 p.value = (sum(s0 >= s) + 0.5) / (N + 1),
+                 data.name = deparse(substitute(x))))
 }
 
-print.exchTest <- function(x, ...)
-{
-  cat("Statistic:", x$statistic,
-      "with p-value", x$pvalue, "\n\n")
-  invisible(x)
-}
-
-
-### Test of exchangeability based on Cn ########################################
-
+##' Test of exchangeability for bivariate copulas based on the
+##' empirical copula -- see SJS paper
+##'
+##' @title Test of exchangeability based on Cn
+##' @param x the data
+##' @param N number of multiplier replications
+##' @param m grid size; if 0, use pseudo-observations
+##' @return an object of class 'htest'
+##' @author Ivan Kojadinovic
 exchTest <- function(x, N = 1000, m = 0)
 {
   ## make pseudo-observations
@@ -81,12 +93,12 @@ exchTest <- function(x, N = 1000, m = 0)
     {
       xis <- yis <- seq(1/m, 1 - 1/m, len = m)
       g <- as.matrix(expand.grid(xis, yis))
-      m <- m^2
+      ng <- m^2
     }
   else
     {
       g <- u
-      m <- n
+      ng <- n
     }
 
   ## compute the test statistic
@@ -96,7 +108,7 @@ exchTest <- function(x, N = 1000, m = 0)
           as.integer(n),
           as.double(g[,1]),
           as.double(g[,2]),
-          as.integer(m),
+          as.integer(ng),
           stat = double(1))$stat
 
   s0 <- .C(exchtestCn,
@@ -105,10 +117,13 @@ exchTest <- function(x, N = 1000, m = 0)
            as.integer(n),
            as.double(g[,1]),
            as.double(g[,2]),
-           as.integer(m),
+           as.integer(ng),
            as.integer(N),
            s0 = double(N))$s0
 
-  structure(class = "exchTest",
-            list(statistic=s, pvalue=(sum(s0 >= s)+0.5)/(N+1)))
+  structure(class = "htest",
+	    list(method = paste("Test of exchangeability for bivariate copulas with argument 'm' set to",m),
+                 statistic = c(statistic = s),
+                 p.value = (sum(s0 >= s) + 0.5) / (N + 1),
+                 data.name = deparse(substitute(x))))
 }
