@@ -14,29 +14,21 @@
 ## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-genFunFrank <- function(copula, u) {
+iPsiFrank <- function(copula, u) {
   alpha <- copula@parameters[1]
   - log( (exp(- alpha * u) - 1) / (exp(- alpha) - 1))
 }
 
-genInvFrank <- function(copula, s) {
+psiFrank <- function(copula, s) {
   alpha <- copula@parameters[1]
   -1/alpha * log(1 + exp(-s) * (exp(-alpha) - 1))
 }
 
-genFunDer1Frank <- function(copula, u) {
-  eval(frankCopula.genfunDer.expr[1], list(u=u, alpha=copula@parameters[1]))
-}
-
-genFunDer2Frank <- function(copula, u) {
-  eval(frankCopula.genfunDer.expr[2], list(u=u, alpha=copula@parameters[1]))
-}
-
-## genInvDerFrank <- function(copula, s, n) {
-##   eval(genInvDerFrank.expr[n + 1], list(s=s, alpha=copula@parameters[1]))
+## psiDerFrank <- function(copula, s, n) {
+##   eval(psiDerFrank.expr[n + 1], list(s=s, alpha=copula@parameters[1]))
 ## }
 
-frankCopula <- function(param, dim = 2L) {
+frankCopula <- function(param = NA_real_, dim = 2L) {
   ## get expressions of cdf and pdf
   cdfExpr <- function(n) {
     expr <-   "- log( (exp(- alpha * u1) - 1) / (exp(- alpha) - 1) )"
@@ -61,18 +53,17 @@ frankCopula <- function(param, dim = 2L) {
   cdf <- cdfExpr(dim)
   if (dim <= 6)  pdf <- pdfExpr(cdf, dim)
   else pdf <- NULL
-  val <- new("frankCopula",
-             dimension = dim,
-             parameters = param[1],
-             exprdist = c(cdf = cdf, pdf = pdf),
-             param.names = "param",
-             param.lowbnd = -Inf,
-             param.upbnd = Inf,
-             message = "Frank copula family; Archimedean copula")
-  val
+  new("frankCopula",
+      dimension = dim,
+      parameters = param[1],
+      exprdist = c(cdf = cdf, pdf = pdf),
+      param.names = "param",
+      param.lowbnd = -Inf,
+      param.upbnd = Inf,
+      fullname = "Frank copula family; Archimedean copula")
 }
 
-rfrankBivCopula <- function(copula, n) {
+rfrankBivCopula <- function(n, copula) {
   val <- cbind(runif(n), runif(n))
   ## to fix numerical rounding problems for alpha >35 but not for alpha < -35
   alpha <- - abs(copula@parameters[1])
@@ -81,26 +72,26 @@ rfrankBivCopula <- function(copula, n) {
   val
 }
 
-## rfrankBivCopula <- function(copula, n) {
+## rfrankBivCopula <- function(n, copula) {
 ##   delta <- copula@parameters[1]
 ##   q <- runif(n); u <- runif(n)
 ##   v <- - 1/ delta * log( 1 - (1 - exp(-delta)) / ((1 / q - 1) * exp(- delta * u) + 1))
 ##   cbind(u, v)
 ## }
 
-rfrankCopula <- function(copula, n) {
+rfrankCopula <- function(n, copula) {
   dim <- copula@dimension
   alpha <- copula@parameters[1]
   if (abs(alpha - 0) < .Machine$double.eps ^ (1/3))
-    return(rcopula(indepCopula(dim), n))
+    return(rCopula(n, indepCopula(dim)))
 ##   if (abs(alpha) <= .Machine$double.eps^.9)
 ##     return (matrix(runif(n * dim), nrow = n))
-  if (dim == 2) return (rfrankBivCopula(copula, n))
+  if (dim == 2) return (rfrankBivCopula(n, copula))
   ## the frailty is a log series distribution with a = 1 - exp(-alpha)
   fr <- rlogseries(n, 1 - exp(-alpha))
   fr <- matrix(fr, nrow = n, ncol = dim)
   val <- matrix(runif(dim * n), nrow = n)
-  genInv(copula, - log(val) / fr)
+  psi(copula, - log(val) / fr)
 }
 
 
@@ -115,7 +106,7 @@ pfrankCopula <- function(copula, u) {
   eval(cdf)
 }
 
-dfrankCopula <- function(copula, u, log=FALSE, ...) {
+dfrankCopula <- function(u, copula, log=FALSE, ...) {
   if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
   pdf <- copula@exprdist$pdf
   dim <- copula@dimension
@@ -131,13 +122,13 @@ dfrankCopula <- function(copula, u, log=FALSE, ...) {
 
 ## dfrankCopula.expr <- function(copula, u) {
 ##   if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
-##   s <- apply(genFunFrank(copula, u), 1, sum)
-##   pdf <- genInvDerFrank(copula, s, copula@dimension) *
-##     apply(genFunDerFrank(copula, u, 1), 1, prod)
+##   s <- apply(iPsiFrank(copula, u), 1, sum)
+##   pdf <- psiDerFrank(copula, s, copula@dimension) *
+##     apply(iPsiDerFrank(copula, u, 1), 1, prod)
 ##   pdf
 ## }
 
-dfrankCopula.pdf <- function(copula, u, log=FALSE) {
+dfrankCopula.pdf <- function(u, copula, log=FALSE) {
   dim <- copula@dimension
   if (dim > 6) stop("Frank copula PDF not implemented for dimension > 6.")
   if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
@@ -149,50 +140,88 @@ dfrankCopula.pdf <- function(copula, u, log=FALSE) {
   else  c(eval(frankCopula.pdf.algr[dim]))
 }
 
-kendallsTauFrankCopula <- function(copula) {
+
+tauFrankCopula <- function(copula) {
   alpha <- copula@parameters[1]
   if (alpha == 0) return (0)
   1 - 4 / alpha * (1 - debye1(alpha))
 }
 
-spearmansRhoFrankCopula <- function(copula) {
+rhoFrankCopula <- function(copula) {
   alpha <- copula@parameters[1]
-  if (alpha == 0) return (0)
-    1 - 12/alpha * (debye1(alpha) - debye2(alpha))
+  if (alpha == 0) 0
+  else
+      ## Genest (1987), "Frank's family of bivariate distributions" (Biometrika, 7, 549--555)
+      1 - 12/alpha * (debye1(alpha) - debye2(alpha))
 }
 
-tailIndexFrankCopula <- function(copula) {
-  c(lower=0, upper=0)
-}
-
-tauDerFrankCopula <- function(copula) {
+dTauFrankCopula <- function(copula) {
   alpha <- copula@parameters
-  return( 4/alpha^2 + 4/(alpha * (exp(alpha) - 1)) - 8/alpha^2 * debye1(alpha) )
+  4/alpha^2 + 4/(alpha * expm1(alpha)) - 8/alpha^2 * debye1(alpha)
 }
 
-rhoDerFrankCopula <- function(copula) {
+dRhoFrankCopula <- function(copula) {
   alpha <- copula@parameters
-  return( 12 / (alpha * (exp(alpha) - 1)) - 36 / alpha^2 * debye2(alpha) + 24 / alpha^2 * debye1(alpha) )
+  12 / (alpha * expm1(alpha)) + (-36 * debye2(alpha) + 24 * debye1(alpha))/ alpha^2
 }
 
 
-setMethod("rcopula", signature("frankCopula"), rfrankCopula)
-setMethod("pcopula", signature("frankCopula"), pfrankCopula)
-setMethod("dcopula", signature("frankCopula"), dfrankCopula.pdf)
+pMatFrank <- function (u, copula, ...) {
+    ## was  pfrankCopula
+    stopifnot(ncol(u) == (d <- copula@dimension))
+    th <- copula@parameters
+    if(d == 2 && !copFrank@paraConstr(th)) # for now, .. to support negative tau
+        pfrankCopula(copula, u=u)
+    else
+        pacopula(u, copFrank, theta=copula@parameters, ...)
+}
 
-setMethod("genFun", signature("frankCopula"), genFunFrank)
-setMethod("genInv", signature("frankCopula"), genInvFrank)
-setMethod("genFunDer1", signature("frankCopula"), genFunDer1Frank)
-setMethod("genFunDer2", signature("frankCopula"), genFunDer2Frank)
-## setMethod("genInvDer", signature("frankCopula"), genInvDerFrank)
+dMatFrank <- function (u, copula, log = FALSE, ...) {
+    ## was  dfrankCopula.pdf
+    stopifnot(ncol(u) == (d <- copula@dimension))
+    th <- copula@parameters
+    if(d == 2 && th < 0) # for now, copFrank does not yet support negative tau
+        dfrankCopula.pdf(u, copula, log=log)
+    else
+        copFrank@dacopula(u, theta=th, log=log, ...)
+}
+setMethod("rCopula", signature("numeric", "frankCopula"), rfrankCopula)
+
+setMethod("pCopula", signature("numeric", "frankCopula"),
+	  function (u, copula, ...)
+	  pMatFrank(matrix(u, ncol = dim(copula)), copula, ...))
+setMethod("pCopula", signature("matrix", "frankCopula"), pMatFrank)
+
+setMethod("dCopula", signature("numeric", "frankCopula"),
+	  function (u, copula, ...)
+	  dMatFrank(matrix(u, ncol = dim(copula)), copula, ...))
+setMethod("dCopula", signature("matrix", "frankCopula"), dMatFrank)
+
+setMethod("iPsi", signature("frankCopula"), iPsiFrank)
+## FIXME {negative tau}
+## setMethod("iPsi", signature("frankCopula"),
+## 	  function(copula, u) copFrank@iPsi(u, theta=copula@parameters))
+setMethod("psi", signature("frankCopula"), psiFrank)
+## FIXME {negative tau}
+## setMethod("psi", signature("frankCopula"),
+## 	  function(copula, s) copFrank@psi(t=s, theta=copula@parameters))
+
+## setMethod("psiDer", signature("frankCopula"), psiDerFrank)
+setMethod("diPsi", signature("frankCopula"),
+	  function(copula, u, degree=1, log=FALSE, ...)
+      {
+	  s <- if(log || degree %% 2 == 0) 1. else -1.
+	  s* copFrank@absdiPsi(u, theta=copula@parameters, degree=degree, log=log, ...)
+      })
 
 
-setMethod("kendallsTau", signature("frankCopula"), kendallsTauFrankCopula)
-setMethod("spearmansRho", signature("frankCopula"), spearmansRhoFrankCopula)
-setMethod("tailIndex", signature("frankCopula"), tailIndexFrankCopula)
 
-setMethod("calibKendallsTau", signature("frankCopula"), calibKendallsTauCopula)
-setMethod("calibSpearmansRho", signature("frankCopula"), calibSpearmansRhoCopula)
+setMethod("tau", signature("frankCopula"), tauFrankCopula)
+setMethod("rho", signature("frankCopula"), rhoFrankCopula)
+setMethod("tailIndex", signature("frankCopula"), function(copula) c(lower=0, upper=0))
 
-setMethod("rhoDer", signature("frankCopula"), rhoDerFrankCopula)
-setMethod("tauDer", signature("frankCopula"), tauDerFrankCopula)
+setMethod("iTau", signature("frankCopula"), iTauCopula)
+setMethod("iRho", signature("frankCopula"), iRhoCopula)
+
+setMethod("dRho", signature("frankCopula"), dRhoFrankCopula)
+setMethod("dTau", signature("frankCopula"), dTauFrankCopula)
