@@ -13,17 +13,38 @@
 ## You should have received a copy of the GNU General Public License along with
 ## this program; if not, see <http://www.gnu.org/licenses/>.
 
-
-### Goodness-of-fit test for extreme value copulas #############################
-
-## copula is a copula of the desired family whose parameters, if necessary,
-## will be used as starting values in fitCopula
-
+##' Goodness-of-fit test for extreme value copulas
+##' See Bernoulli 2011
+##'
+##' @title Goodness-of-fit test for extreme value copulas
+##' @param copula a copula of the desired family
+##' @param x the data
+##' @param N the number of parameteric bootstrap iterations
+##' @param method parameter estimation method
+##' @param estimator nonparametric estimator of the Pickands dependence function
+##' @param m grid size
+##' @param verbose display progress bar if TRUE
+##' @param print.every is deprecated
+##' @param optim.method for fitCopula
+##' @return an object of class 'htest'
+##' @author Ivan Kojadinovic
 gofEVCopula <- function(copula, x, N = 1000, method = "mpl",
-                        estimator = "CFG", m = 1000, print.every = 100,
+                        estimator = "CFG", m = 1000,
+                        verbose = TRUE, print.every = NULL,
                         optim.method = "Nelder-Mead")
 {
     n <- nrow(x)
+    p <- ncol(x)
+
+    if (n < 2) stop("There should be at least 2 observations")
+
+    if (copula@dimension != 2 || p != 2)
+      stop("The copula and the data should be of dimension two")
+
+    if (!is.null(print.every)) {
+        warning("Argument 'print.every' is deprecated. Please use 'verbose' instead.")
+        verbose <- print.every > 0
+    }
 
     ## make pseudo-observations
     u <- apply(x,2,rank)/(n+1)
@@ -32,7 +53,7 @@ gofEVCopula <- function(copula, x, N = 1000, method = "mpl",
     fcop <- fitCopula(copula, u, method, estimate.variance=FALSE,
                       optim.method=optim.method)@copula
 
-    ## where to compute Afun
+    ## where to compute A
     g <- seq(0,1-1/m,by=1/m)
 
     ## compute the test statistic
@@ -41,19 +62,19 @@ gofEVCopula <- function(copula, x, N = 1000, method = "mpl",
             as.integer(m),
             as.double(-log(u[,1])),
             as.double(-log(u[,2])),
-            as.double(Afun(fcop,g)),
+            as.double(A(fcop,g)),
             stat = double(2),
             as.integer(estimator == "CFG"))$stat
 
     ## simulation of the null distribution
     s0 <- matrix(NA, N, 2)
-    if (print.every > 0)
-      cat(paste("Progress will be displayed every", print.every, "iterations.\n"))
+    if (verbose) {
+	pb <- txtProgressBar(max = N, style = 3) # setup progress bar
+	on.exit(close(pb)) # and close it on exit
+    }
     for (i in 1:N)
-      {
-        if (print.every > 0 & i %% print.every == 0)
-          cat(paste("Iteration",i,"\n"))
-        u0 <- apply(rcopula(fcop,n),2,rank)/(n+1)
+    {
+        u0 <- apply(rCopula(n, fcop),2,rank)/(n+1)
 
         ## fit the copula
         fcop0 <-  fitCopula(copula, u0, method, estimate.variance=FALSE,
@@ -64,28 +85,28 @@ gofEVCopula <- function(copula, x, N = 1000, method = "mpl",
                      as.integer(m),
                      as.double(-log(u0[,1])),
                      as.double(-log(u0[,2])),
-                     as.double(Afun(fcop0,g)),
+                     as.double(A(fcop0,g)),
                      stat = double(2),
                      as.integer(estimator == "CFG"))$stat
-      }
+
+        if (verbose) setTxtProgressBar(pb, i) # update progress bar
+    }
 
     ## corrected version only
-    gof <- list(statistic=s[1],
-                pvalue=(sum(s0[,1] >= s[1])+0.5)/(N+1),
-                parameters=fcop@parameters)
+    structure(class = "htest",
+              list(method = paste("Parametric bootstrap based GOF test for EV copulas with argument 'method' set to '",
+                   method, "' and argument 'estimator' set to '", estimator, "'", sep = ""),
+                   parameter = c(parameter = fcop@parameters),
+                   statistic = c(statistic = s[1]),
+                   p.value=(sum(s0[,1] >= s[1])+0.5)/(N+1),
+                   data.name = deparse(substitute(x))))
 
-    class(gof) <- "gofCopula"
-    gof
 }
 
-################################################################################
 
-## version for simulations
-## was named gofEVCopula before
-## not exported
-
-gofAfun <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
-                    m = 1000, print.every = 100, optim.method = "Nelder-Mead")
+### Version for simulations; was named gofEVCopula before; not exported
+gofA <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
+                    m = 1000, verbose = TRUE, optim.method = "Nelder-Mead")
 {
     n <- nrow(x)
     p <- ncol(x)
@@ -102,10 +123,10 @@ gofAfun <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
               as.integer(n),
               as.integer(p),
               as.double(u),
-              as.double(pcopula(fcop,u)),
+              as.double(pCopula(u,fcop)),
               stat = double(1))$stat
 
-    ## where to compute Afun
+    ## where to compute A
     g <- seq(0,1-1/m,by=1/m)
 
     ## compute the CFG test statistic
@@ -114,9 +135,9 @@ gofAfun <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
                as.integer(m),
                as.double(-log(u[,1])),
                as.double(-log(u[,2])),
-               as.double(Afun(fcop,g)),
+               as.double(A(fcop,g)),
                stat = double(2),
-               as.integer(1)# estimator == "CFG"
+               as.integer(1)            # estimator == "CFG"
                )$stat
     ## compute the Pickands test statistic
     sPck <- .C(cramer_vonMises_Afun,
@@ -124,25 +145,25 @@ gofAfun <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
                as.integer(m),
                as.double(-log(u[,1])),
                as.double(-log(u[,2])),
-               as.double(Afun(fcop,g)),
+               as.double(A(fcop,g)),
                stat = double(2),
-               as.integer(0) # estimator == Pickard
+               as.integer(0)            # estimator == Pickands
                )$stat
 
     s <- c(sCn, sCFG, sPck)
 
     ## simulation of the null distribution
     s0 <- matrix(NA, N, 5)
-    if (print.every > 0)
-      cat(paste("Progress will be displayed every", print.every, "iterations.\n"))
+    if (verbose) {
+	pb <- txtProgressBar(max = N, style = 3) # setup progress bar
+	on.exit(close(pb)) # and close it on exit
+    }
 
     ## set starting values for fitCopula
     copula@parameters <- fcop@parameters
     for (i in 1:N)
-      {
-        if (print.every > 0 & i %% print.every == 0)
-          cat(paste("Iteration",i,"\n"))
-        u0 <- apply(rcopula(fcop,n),2,rank)/(n+1)
+    {
+        u0 <- apply(rCopula(n, fcop),2,rank)/(n+1)
 
         ## fit the copula
         fcop0 <-  fitCopula(copula, u0, method, estimate.variance=FALSE,
@@ -152,7 +173,7 @@ gofAfun <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
                    as.integer(n),
                    as.integer(p),
                    as.double(u0),
-                   as.double(pcopula(fcop0,u0)),
+                   as.double(pCopula(u0, fcop0)),
                    stat = double(1))$stat
 
         sCFG0 <-  .C(cramer_vonMises_Afun,
@@ -160,21 +181,23 @@ gofAfun <- function(copula, x, N = 1000, method = "mpl", # estimator = "CFG",
                      as.integer(m),
                      as.double(-log(u0[,1])),
                      as.double(-log(u0[,2])),
-                     as.double(Afun(fcop0,g)),
+                     as.double(A(fcop0,g)),
                      stat = double(2),
-                     as.integer(1) # estimator == "CFG"
+                     as.integer(1)      # estimator == "CFG"
                      )$stat
         sPck0 <-  .C(cramer_vonMises_Afun,
                      as.integer(n),
                      as.integer(m),
                      as.double(-log(u0[,1])),
                      as.double(-log(u0[,2])),
-                     as.double(Afun(fcop0,g)),
+                     as.double(A(fcop0,g)),
                      stat = double(2),
-                     as.integer(0) # estimator == Pickard
+                     as.integer(0)      # estimator == Pickands
                      )$stat
         s0[i,] <- c(sCn0, sCFG0, sPck0)
-      }
+
+        if (verbose) setTxtProgressBar(pb, i) # update progress bar
+    }
 
     list(statistic = s,
          pvalue = sapply(1:5, function(i) (sum(s0[,i] >= s[i])+0.5)/(N+1)),
