@@ -14,7 +14,7 @@
 ## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-AfunTev <- function(copula, w) {
+ATev <- function(copula, w) {
   rho <- copula@parameters[1]
   ## nu <- getdf(copula) ## defined in tCopula.R
   nu <- copula@df
@@ -25,7 +25,7 @@ AfunTev <- function(copula, w) {
   ifelse(w == 0 | w == 1, 1, A)
 }
 
-AfunDerTev <- function(copula, w) {
+dAduTev <- function(copula, w) {
   rho <- copula@parameters[1]
   ## nu <- getdf(copula) ## defined in tCopula.R
   nu <- copula@df
@@ -54,7 +54,7 @@ AfunDerTev <- function(copula, w) {
   data.frame(der1 = der1, der2 = der2)
 }
 
-tevCopula <- function(param, df = 4, df.fixed = FALSE) {
+tevCopula <- function(param = NA_real_, df = 4, df.fixed = FALSE) {
   dim <- 2L
   pdim <- length(param)
   parameters <- param
@@ -75,26 +75,26 @@ tevCopula <- function(param, df = 4, df.fixed = FALSE) {
              param.names = param.names,
              param.lowbnd = param.lowbnd,
              param.upbnd = param.upbnd,
-             message = paste("t-EV copula family",
+             fullname = paste("t-EV copula family",
                if(df.fixed) paste("df fixed at", df)))
 }
 
-ptevCopula <- function(copula, u) {
+ptevCopula <- function(u, copula) {
   dim <- copula@dimension
   if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
   ## for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
   u1 <- u[,1]; u2 <- u[,2]
   logu <- log(u1 * u2)
-  exp(logu * AfunTev(copula, log(u2) / logu))
+  exp(logu * ATev(copula, log(u2) / logu))
 }
 
-dtevCopula <- function(copula, u, log=FALSE, ...) {
+dtevCopula <- function(u, copula, log=FALSE, ...) {
   dim <- copula@dimension
   if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
   ## for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
   u1 <- u[,1]; u2 <- u[,2]
 
-  C <- ptevCopula(copula, u)
+  C <- ptevCopula(u, copula)
   logu <- log(u1 * u2)
   w <- log(u2) / logu
   wexpr <- ~ log(u2) / log(u1 * u2)
@@ -102,8 +102,8 @@ dtevCopula <- function(copula, u, log=FALSE, ...) {
   dwdu1 <- c(attr(dw, "gradient")[,"u1"])
   dwdu2 <- c(attr(dw, "gradient")[,"u2"])
   d2wdu1du2 <- c(attr(dw, "hessian")[,"u1","u2"])
-  A <- AfunTev(copula, w)
-  Ader <- AfunDerTev(copula, w)
+  A <- ATev(copula, w)
+  Ader <- dAduTev(copula, w)
   Ader1 <- Ader$der1; Ader2 <- Ader$der2
   dCdu1 <- C * (1 / u1 * A + logu * Ader1 * dwdu1)
   dCdu2 <- C * (1 / u2 * A + logu * Ader1 * dwdu2)
@@ -116,19 +116,19 @@ dtevCopula <- function(copula, u, log=FALSE, ...) {
   if(log) log(pdf) else pdf
 }
 
-derCdfWrtArgsSymEvCopula <- function(cop, u) {
+dCduSymEvCopula <- function(cop, u) {
   mat <- matrix(NA, nrow(u), 2)
-  pcop <- pcopula(cop, u)
+  pcop <- pCopula(u, cop)
   loguv <- log(u[,1]) + log(u[,2])
   w <- log(u[,2]) / loguv
-  a <- Afun(cop, w)
-  aDer <- AfunDer(cop, w)$der1
+  a <- A(cop, w)
+  aDer <- dAdu(cop, w)$der1
   mat[,1] <- pcop * (a / u[,1] - loguv * aDer * log(u[,2]) / (loguv)^2 / u[,1])
   mat[,2] <- pcop * (a / u[,2] + loguv * aDer * log(u[,1]) / (loguv)^2 / u[,2])
   mat
 }
 
-setMethod("derCdfWrtArgs", signature("tevCopula"), derCdfWrtArgsSymEvCopula)
+setMethod("dCdu", signature("tevCopula"), dCduSymEvCopula)
 
 
 ## This block is copied from ../../copulaUtils/assoc/ ##########################
@@ -144,12 +144,12 @@ tevTauFun <- function(alpha) {
   valFun(theta)
 }
 
-kendallsTauTevCopula <- function(copula) {
+tauTevCopula <- function(copula) {
   alpha <- copula@parameters[1]
   tevTauFun(alpha)
 }
 
-calibKendallsTauTevCopula <- function(copula, tau) {
+iTauTevCopula <- function(copula, tau) {
   if (any(tau < 0)) warning("tau is out of the range [0, 1]")
   tevTauInv <- approxfun(x = .tevTau$assoMeasFun$fm$ysmth,
                          y = .tevTau$assoMeasFun$fm$x, rule=2)
@@ -169,7 +169,7 @@ tevTauDer <- function(alpha) {
   valFun(theta, 1) * forwardDer(alpha, ss)
 }
 
-tauDerTevCopula <- function(copula) {
+dTauTevCopula <- function(copula) {
   alpha <- copula@parameters[1]
   tevTauDer(alpha)
 }
@@ -185,12 +185,12 @@ tevRhoFun <- function(alpha) {
   valFun(theta)
 }
 
-spearmansRhoTevCopula <- function(copula) {
+rhoTevCopula <- function(copula) {
   alpha <- copula@parameters[1]
   tevRhoFun(alpha)
 }
 
-calibSpearmansRhoTevCopula <- function(copula, rho) {
+iRhoTevCopula <- function(copula, rho) {
   if (any(rho < 0)) warning("rho is out of the range [0, 1]")
   tevRhoInv <- approxfun(x = .tevRho$assoMeasFun$fm$ysmth,
                          y = .tevRho$assoMeasFun$fm$x, rule = 2)
@@ -210,24 +210,27 @@ tevRhoDer <- function(alpha) {
   valFun(theta, 1) * forwardDer(alpha, ss)
 }
 
-rhoDerTevCopula <- function(copula) {
+dRhoTevCopula <- function(copula) {
   alpha <- copula@parameters[1]
   tevRhoDer(alpha)
 }
 
 ################################################################################
 
-setMethod("pcopula", signature("tevCopula"), ptevCopula)
-setMethod("dcopula", signature("tevCopula"), dtevCopula)
+setMethod("pCopula", signature("numeric", "tevCopula"),ptevCopula)
+setMethod("pCopula", signature("matrix", "tevCopula"), ptevCopula)
+setMethod("dCopula", signature("numeric", "tevCopula"),dtevCopula)
+setMethod("dCopula", signature("matrix", "tevCopula"), dtevCopula)
 
-setMethod("Afun", signature("tevCopula"), AfunTev)
-setMethod("AfunDer", signature("tevCopula"), AfunDerTev)
 
-setMethod("kendallsTau", signature("tevCopula"), kendallsTauTevCopula)
-setMethod("spearmansRho", signature("tevCopula"), spearmansRhoTevCopula)
+setMethod("A", signature("tevCopula"), ATev)
+setMethod("dAdu", signature("tevCopula"), dAduTev)
 
-setMethod("calibKendallsTau", signature("tevCopula"), calibKendallsTauTevCopula)
-setMethod("calibSpearmansRho", signature("tevCopula"), calibSpearmansRhoTevCopula)
+setMethod("tau", signature("tevCopula"), tauTevCopula)
+setMethod("rho", signature("tevCopula"), rhoTevCopula)
 
-setMethod("tauDer", signature("tevCopula"), tauDerTevCopula)
-setMethod("rhoDer", signature("tevCopula"), rhoDerTevCopula)
+setMethod("iTau", signature("tevCopula"), iTauTevCopula)
+setMethod("iRho", signature("tevCopula"), iRhoTevCopula)
+
+setMethod("dTau", signature("tevCopula"), dTauTevCopula)
+setMethod("dRho", signature("tevCopula"), dRhoTevCopula)
