@@ -40,8 +40,8 @@ fCols <- colorRampPalette(c("red", "white", "blue"), space = "Lab")
 
 
 ## frankCopula
-theta.fr <- calibKendallsTau(frankCopula(0), tau)
-dcop <- matrix(dcopula(frankCopula(param=theta.fr, dim = 2), umat),
+theta.fr <- iTau(frankCopula(), tau)
+dcop <- matrix(dCopula(umat, frankCopula(param=theta.fr, dim = 2)),
                n1, n2)
 filled.contour(u1, u2, dcop, color.palette = fCols,
                main=sprintf("Density( frankCopula(%.4g) )", theta.fr))
@@ -49,9 +49,9 @@ round(dcop, 3)
 
 
 ## claytonCopula
-(theta.cl <- calibKendallsTau(claytonCopula(1), tau))
+(theta.cl <- iTau(claytonCopula(), tau))
 stopifnot(all.equal(theta.cl, copClayton@tauInv(tau), tol = 1e-13))
-dcop <- matrix(dcopula(claytonCopula(param=theta.cl, dim = 2), umat),
+dcop <- matrix(dCopula(umat, claytonCopula(param=theta.cl, dim = 2)),
                n1, n2)
 filled.contour(u1, u2, dcop, color.palette = fCols,
                main=sprintf(" Density( claytonCopula(%.4g) )", theta.cl))
@@ -62,9 +62,9 @@ round(dcop, 3)
 
 
 ## gumbelCopula
-theta.gu <- calibKendallsTau(gumbelCopula(1), tau)
+theta.gu <- iTau(gumbelCopula(), tau)
 stopifnot(all.equal(theta.gu, copGumbel@tauInv(tau), tol = 1e-13))
-dcop <- matrix(dcopula(gumbelCopula(param=theta.gu, dim = 2), umat),
+dcop <- matrix(dCopula(umat, gumbelCopula(param=theta.gu, dim = 2)),
                n1, n2)
 filled.contour(u1, u2, dcop, color.palette = fCols,
                main=sprintf("Density( gumbelCopula(%.4g) )", theta.gu))
@@ -75,10 +75,10 @@ round(dcop, 3)
 
 ## normalCopula
 uB <- cbind(c(0:1, .5), (1:9)/10) # values at boundaries
-fC <- dcopula(normalCopula(0.55), uB)
+fC <- dCopula(uB, normalCopula(0.55))
 stopifnot(is.finite(fC), length(fC)==nrow(uB), fC[-3*(1:3)] == 0)
-theta.n <- calibKendallsTau(normalCopula(0), tau)
-dcop <- matrix(dcopula(normalCopula(param=theta.n, dim = 2), umat),
+theta.n <- iTau(normalCopula(), tau)
+dcop <- matrix(dCopula(umat, normalCopula(param=theta.n, dim = 2)),
                n1, n2)
 filled.contour(u1, u2, log(dcop), color.palette = fCols,
                main=sprintf("log Density( normalCopula(%.4g) )", theta.n))
@@ -88,10 +88,10 @@ round(dcop, 3)
 
 
 ## tCopula
-fC <- dcopula(tCopula(0.55), uB)
+fC <- dCopula(uB, tCopula(0.55))
 stopifnot(is.finite(fC), length(fC)==nrow(uB), fC[-3*(1:3)] == 0)
-(theta.t. <- calibKendallsTau(tCopula(0, df=10), tau))
-dcop <- matrix(dcopula(tCopula(param=theta.t., dim = 2, df=10), umat),
+(theta.t. <- iTau(tCopula(df=10), tau))
+dcop <- matrix(dCopula(umat, tCopula(param=theta.t., dim = 2, df=10)),
                n1, n2)
 filled.contour(u1, u2, log(dcop), color.palette = fCols,
                main=sprintf("log Density( tCopula(%.4g, df = 10) )", theta.t.))
@@ -99,7 +99,7 @@ filled.contour(u1, u2, dcop, color.palette = fCols,
                main=sprintf("Density( tCopula(%.4g, df = 10) )", theta.t.))
 round(dcop, 3)
 ## tCopula -- df=4
-(theta <- calibKendallsTau(tCopula(0, df=4), tau))
+(theta <- iTau(tCopula(df=4), tau))
 dcop <- matrix(dcopula(tCopula(param=theta, dim = 2, df=4), umat),
                n1, n2)
 filled.contour(u1, u2, log(dcop), color.palette = fCols,
@@ -110,9 +110,9 @@ round(dcop, 3)
 
 ## galambosCopula
 #
-(theta <- calibKendallsTau(galambosCopula(1), tau))
-stopifnot(all.equal(tau, kendallsTau(galambosCopula(theta)), tol = 1e-5))
-dcop <- matrix(dcopula(galambosCopula(param=theta), umat),
+(theta <- iTau(galambosCopula(), tau))
+stopifnot(all.equal(tau, tau(galambosCopula(theta)), tol = 1e-5))
+dcop <- matrix(dCopula(umat, galambosCopula(param=theta)),
                n1, n2)
 filled.contour(u1, u2, log(dcop), color.palette = fCols,
                main=sprintf("log Density( galambosCopula(%.4g) )", theta))
@@ -123,9 +123,9 @@ round(dcop, 3)
 
 ## amhCopula
 tau <- 0.3 ## -- to be in range for AMH
-(theta <- calibKendallsTau(amhCopula(1), tau))
-stopifnot(all.equal(tau, kendallsTau(amhCopula(theta)), tol = 1e-5))
-dcop <- matrix(dcopula(amhCopula(param=theta), umat),
+(theta <- iTau(amhCopula(), tau))
+stopifnot(all.equal(tau, tau(amhCopula(theta)), tol = 1e-5))
+dcop <- matrix(dCopula(umat, amhCopula(param=theta)),
                n1, n2)
 filled.contour(u1, u2, dcop, color.palette = fCols,
                main=sprintf("Density( amhCopula(%.4g) )", theta))
